// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library omnitest;

import 'dart:io';
import 'dart:async';

import 'package:test/test.dart';
import 'package:omnibus/patterns.dart';
import 'package:omnitest/data.dart';
import 'package:omnitest/test.dart';
import 'package:omnitest/exceptions.dart';

/// Function that is passed down to the test framework.
typedef Future OmnitestFunction();

/// Create test function
///
/// The returned function can be passed to the test function from the test
/// package. The function will execute all tests that are described in the
/// [testFile] against the [module].
OmnitestFunction omnitest(ModuleMain module, String testFile) {
  return () async {
    // Parse YAML config.
    OmnitestFile tests;
    try {
      tests = new OmnitestFile();
      await tests.parseYaml(new File(testFile).readAsStringSync());
    } on OmnitestException catch (e) {
      fail(e.message);
    }

    // Run tests.
    await runTests(tests, module);
  };
}
