// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// Data of a single message in a use case message chain
class OmnitestChainMessage {
  /// Expect YAML key
  static const expectKey = 'expect';

  /// Message data
  OmnibusMessage data;

  /// Expected activity
  OmnitestExpect expect;

  /// Load data from a YAML map.
  OmnitestChainMessage.fromYaml(
      YamlMap yamlMap, Map<String, OmnibuildMessage> messageSpec) {
    var omnibusId = getOmnibusId(messageSpec.keys, yamlMap);
    if (omnibusId != null) {
      // This cannot be a response itself (responses cannot be in the message
      // chain).
      if (messageSpec[omnibusId].type == OmnibusMessageType.response) {
        throw new OmnitestException.yaml(
            'A message-chain cannot contain responses.');
      } else {
        // Construct message data.
        data = messageSpec[omnibusId].type == OmnibusMessageType.event
            ? new OmnitestEvent(
                omnibusId, resolveMessageData(yamlMap, omnibusId))
            : new OmnitestRequest(
                omnibusId, resolveMessageData(yamlMap, omnibusId));

        // Parse expected messages activity.
        if (yamlMap.containsKey(expectKey) && yamlMap[expectKey] is YamlList) {
          expect = new OmnitestExpect.fromYaml(yamlMap[expectKey], messageSpec);
        }
      }
    } else {
      throw new OmnitestException.noOmnibusIdKey(yamlMap);
    }
  }
}
