// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// Expected request that is published as result of a message in a use case
/// message chain.
class OmnitestExpectedRequest {
  /// Module responses YAML key
  static const moduleResponsesKey = 'module-responses';

  /// Mocked responses YAML key
  static const mockedResponsesKey = 'mocked-responses';

  /// Request data
  OmnitestRequest data;

  /// Request responses that should be provided by the module.
  List<OmnitestResponse> moduleResponses = new List<OmnitestResponse>();

  /// Request responses that should be mocked by omnitest.
  List<OmnitestResponse> mockedResponses = new List<OmnitestResponse>();

  /// Expected HTTP requests
  List<OmnitestHttpRequest> expectedHttpRequests =
      new List<OmnitestHttpRequest>();

  /// Load data from a YAML map.
  OmnitestExpectedRequest.fromYaml(String omnibusId, YamlMap yamlData,
      Map<String, OmnibuildMessage> messageSpec) {
    // Construct message data.
    data =
        new OmnitestRequest(omnibusId, resolveMessageData(yamlData, omnibusId));

    // Common function for processing module and mocked responses.
    void processResponses(YamlList list, bool mocked) {
      list.forEach((dynamic item) {
        if (item is YamlMap) {
          YamlMap yamlMap = item;
          String omnibusId = getOmnibusId(messageSpec.keys, yamlMap);
          if (omnibusId != null) {
            // Validate that this is a response message.
            if (messageSpec[omnibusId].type == OmnibusMessageType.response) {
              // Construct message.
              var message = new OmnitestResponse(
                  omnibusId, resolveMessageData(yamlMap, omnibusId));

              // Add message to $moduleResponses or $mockedResponses.
              if (mocked) {
                mockedResponses.add(message);
              } else {
                moduleResponses.add(message);
              }
            } else {
              throw new OmnitestException.yaml(
                  'Only response message are allowed in ${mocked ? 'mocked' : 'module'}-messages.');
            }
          } else {
            throw new OmnitestException.noOmnibusIdKey(yamlMap);
          }
        } else if (item is String) {
          expectedHttpRequests.add(new OmnitestHttpRequest.fromString(item));
        }
      });
    }

    // Loop through module responses.
    if (yamlData.containsKey(moduleResponsesKey) &&
        yamlData[moduleResponsesKey] is YamlList) {
      processResponses(yamlData[moduleResponsesKey], false);
    }

    // Loop through mocked responses.
    if (yamlData.containsKey(mockedResponses) &&
        yamlData[mockedResponses] is YamlList) {
      processResponses(yamlData[mockedResponses], true);
    }
  }
}
