// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// Event logging handler
class OmnitestEventLogger extends OmnibusEventHandler {
  /// Event log
  List<OmnitestEvent> log = new List<OmnitestEvent>();

  /// Constructor
  OmnitestEventLogger() : super(omnibusGlobalOmnibusId, new EmptyState());

  /// Process incoming event.
  Future<List<Future<Null>>> process(OmnibusEvent event) async {
    log.add(new OmnitestEvent(event.omnibusId, event.toPds()));
    return [];
  }
}

/// Universal global request handler callback
typedef Future<List<OmnibusResponse>> GlobalReqHandlerCb(
    OmnibusRequest request);

/// Universal global request handler
class OmnitestGlobalRequestHandler extends OmnibusRequestHandler {
  /// Request log
  List<OmnitestRequest> log = new List<OmnitestRequest>();

  /// Skip one cycle
  bool _skipOnce = false;

  /// Callback
  GlobalReqHandlerCb _callback;

  /// Constructor
  OmnitestGlobalRequestHandler(this._callback)
      : super(omnibusGlobalOmnibusId, new EmptyState());

  /// Skip one cycle.
  void skipOnce() {
    _skipOnce = true;
  }

  /// Process incoming event.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest request) async {
    if (!_skipOnce) {
      // Update log.
      log.add(new OmnitestRequest(request.omnibusId, request.toPds()));

      // Get customized responses from the callback.
      List<OmnibusResponse> responses = await _callback(request);

      // Futurize
      var futureResponses = new List<Future<OmnibusResponse>>();
      responses.forEach((OmnibusResponse response) =>
          futureResponses.add(new Future<OmnibusResponse>.value(response)));
      return futureResponses;
    } else {
      _skipOnce = false;
      return [];
    }
  }
}
