// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// Expected HTTP request
class OmnitestHttpRequest {
  /// Request method
  final String method;

  /// Request URL
  final String url;

  /// Constuctor
  OmnitestHttpRequest(this.method, this.url);

  /// Initialize from a String representation.
  /// Format: `$METHOD $url`
  factory OmnitestHttpRequest.fromString(String str) {
    String method = str.substring(0, str.indexOf(' '));
    String url = str.substring(str.indexOf(' ')).trimLeft();
    return new OmnitestHttpRequest(method, url);
  }

  /// Hash code
  int get hashCode => hash2(method, url);

  /// Compare
  bool operator ==(m) =>
      m is OmnitestHttpRequest && method == m.method && url == m.url;
}

/// HTTP request logging using http.MockClient
class HttpLogger extends http.IOClient {
  /// HTTP log
  List<OmnitestHttpRequest> log = new List<OmnitestHttpRequest>();

  // Constructor
  HttpLogger() : super();

  // Send
  Future<http.StreamedResponse> send(http.BaseRequest request) {
    log.add(new OmnitestHttpRequest(request.method, request.url.toString()));
    return super.send(request);
  }
}
