// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// Test if all elements in the first set are also contained in the second set.
bool orderlessListIsSuperlist(List list, List superlist) {
  // Find a unique element in $superlist for each element in $list.
  var usedIndices = new Set<int>();
  for (var i = 0; i < list.length; i++) {
    var superlistIdx = superlist.indexOf(list[i]);
    if (superlistIdx != -1) {
      while (usedIndices.contains(superlistIdx)) {
        // Find an alternative index if the first index is already used for a
        // previous element (this is the case if there are multiple equal
        // elements).
        superlistIdx = superlist.indexOf(list[i], superlistIdx + 1);

        // If an alternative index does not exist the two lists are not equal.
        if (superlistIdx == -1) {
          return false;
        }
      }
      usedIndices.add(superlistIdx);
    } else {
      // No index for this element could be found.
      return false;
    }
  }
  return true;
}

/// Compare two lists without taking the element sequence into consideration.
bool orderlessListCompare(List a, List b) {
  // If the lengths are not equal the lists cannot be equal either.
  if (a.length != b.length) {
    return false;
  } else {
    return orderlessListIsSuperlist(a, b);
  }
}
