// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// Base class for Omnitest flavoured Omnibus messages.
abstract class OmnitestMessageBase {
  /// PDS
  Map<String, dynamic> pds;

  /// To PDS
  Map<String, dynamic> toPds() => pds;
}

/// Omnitest event implementation
class OmnitestEvent extends OmnibusEvent with OmnitestMessageBase {
  OmnitestEvent(String omnibusId, Map<String, dynamic> _pds)
      : super(omnibusId) {
    pds = _pds;
  }

  /// Hash code
  int get hashCode => hash2(omnibusId, JSON.encode(pds));

  /// Compare
  bool operator ==(m) =>
      m is OmnibusMessage &&
      omnibusId == m.omnibusId &&
      type == m.type &&
      new DeepCollectionEquality().equals(pds, m.toPds());
}

/// Omnitest request implementation
class OmnitestRequest extends OmnibusRequest with OmnitestMessageBase {
  OmnitestRequest(String omnibusId, Map<String, dynamic> _pds)
      : super(omnibusId) {
    pds = _pds;
  }

  /// Hash code
  int get hashCode => hash2(omnibusId, JSON.encode(pds));

  /// Compare
  bool operator ==(m) =>
      m is OmnibusMessage &&
      omnibusId == m.omnibusId &&
      type == m.type &&
      new DeepCollectionEquality().equals(pds, m.toPds());
}

/// Omnitest response implementation
class OmnitestResponse extends OmnibusResponse with OmnitestMessageBase {
  OmnitestResponse(String omnibusId, Map<String, dynamic> _pds)
      : super(omnibusId) {
    pds = _pds;
  }

  /// Hash code
  int get hashCode => hash2(omnibusId, JSON.encode(pds));

  /// Compare
  bool operator ==(m) =>
      m is OmnibusMessage &&
      omnibusId == m.omnibusId &&
      type == m.type &&
      new DeepCollectionEquality().equals(pds, m.toPds());
}
