// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// These keys cannot be used as omnibusId for messages that are tested
/// with this program (seriously, why would you ever use these?).
const reservedKeys = const ['expect', 'module-responses', 'mocked-responses'];

/// Get the first key in $map that can be identified as omnibusId given that
/// $ids contains all omnibusId's that can be used.
String getOmnibusId(Iterable<String> allIds, YamlMap map) {
  var ids = map.keys.where((String key) => allIds.contains(key));
  return ids.length == 1 ? ids.first : null;
}

/// Get message data from the given YamlNode using the given omnibusId. Message
/// data can be directly located under the omnibusId key, but it can also be
/// located in a separate file that is linked using a path string.
Map<String, dynamic> resolveMessageData(YamlMap map, String omnibusId) {
  // First try to parse as map.
  if (map[omnibusId] is YamlMap) {
    return pdsFromMap(map[omnibusId] as Map<String, dynamic>);
  } else if (map[omnibusId] is String) {
    // Try to read this file path.
    try {
      var fileContent = new File(map[omnibusId]).readAsStringSync();
      var yamlContent = loadYaml(fileContent);
      if (yamlContent is YamlMap) {
        return pdsFromMap(yamlContent as Map<String, dynamic>);
      } else {
        return new Map<String, dynamic>();
      }
    } catch (e) {
      return new Map<String, dynamic>();
    }
  } else {
    return new Map<String, dynamic>();
  }
}
