// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.test;

/// Typedef for _OmnitestFabric._processRequest
typedef List<OmnibusResponse> ProcessRequest(
    OmnitestRequest request, List<OmnitestResponse> moduleResponses);

/// Omnitest testing fabric
class OmnitestFabric {
  /// Primary bus
  Omnibus bus;

  /// Secondary bus
  Omnibus secondBus;

  /// Event logger
  OmnitestEventLogger eventLogger = new OmnitestEventLogger();

  /// Request handler
  OmnitestGlobalRequestHandler requestHandler;

  /// HTTP request logger
  HttpLogger httpLogger = new HttpLogger();

  /// Request processing callback
  ProcessRequest _processRequest;

  /// Constructor
  OmnitestFabric(this._processRequest) {
    requestHandler = new OmnitestGlobalRequestHandler(_globalRequestHandler);
    bus = new Omnibus();
    bus.bind(eventLogger);
    bus.bind(requestHandler);
    secondBus = new Omnibus();
  }

  /// Clear event, request and HTTP request log
  void clearLogs() {
    eventLogger.log.clear();
    requestHandler.log.clear();
    httpLogger.log.clear();
  }

  /// Check logs againts expected activity.
  void check(OmnitestExpect expect) {
    // Check event log.
    if (!orderlessListCompare(expect.expectedEvents, eventLogger.log)) {
      throw new OmnitestException.unexpectedEvents(
          expect.expectedEvents, eventLogger.log);
    }

    // Check request log.
    if (!orderlessListCompare(
        expect.getExpectedRequests(), requestHandler.log)) {
      throw new OmnitestException.unexpectedRequests(
          expect.getExpectedRequests(), requestHandler.log);
    }

    // Check HTTP request log.
    if (!orderlessListCompare(expect.expectedHttpRequests, httpLogger.log)) {
      throw new OmnitestException.unexpectedHttpRequests(
          expect.expectedHttpRequests, httpLogger.log);
    }
  }

  /// Global request handler
  Future<List<OmnibusResponse>> _globalRequestHandler(
      OmnibusRequest request) async {
    // Check what responses are provided by the module itself.
    List<OmnibusResponse> moduleResponses =
        await (await secondBus.publishRequest(request)).onComplete();

    // Convert all responses to OmnitestResponse to make them comparable.
    var omnitestResponses = new List<OmnitestResponse>();
    moduleResponses.forEach((OmnibusResponse response) => omnitestResponses
        .add(new OmnitestResponse(response.omnibusId, response.toPds())));

    // Return mocked responses.
    return _processRequest(
        new OmnitestRequest(request.omnibusId, request.toPds()),
        omnitestResponses);
  }
}
