// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

library omnitest.test.calculator;

import 'dart:async';

import 'package:http/http.dart' as http;
import 'package:omnibus/omnibus.dart';
import 'package:omnibus/patterns.dart';

import '../messages/messages.dart' as v0;

/// Calculator module main function
///
/// Binds all calculation handlers to the Omnibus.
Future main(Omnibus bus, http.Client httpClient) async {
  bus.bind(new v0.CalculationRequestHandler(
      calcAddHandler, new SimpleState(bus, httpClient),
      op: v0.CalculationOperator.add));
  bus.bind(new v0.CalculationRequestHandler(
      calcSubtractHandler, new EmptyState(),
      op: v0.CalculationOperator.subtract));
  bus.bind(new v0.CalculationRequestHandler(
      calcMultiplyHandler, new EmptyState(),
      op: v0.CalculationOperator.multiply));
  bus.bind(new v0.CalculationRequestHandler(calcDivideHandler, new EmptyState(),
      op: v0.CalculationOperator.divide));

  // Do addition request so we can test expect-onload.
  await bus.request(new v0.CalculationRequest());
}

/// Addition calculator
Future<v0.CalculationResponse> calcAddHandler(
    v0.CalculationRequest r, SimpleState state) async {
  // Do GET request to http://fake.api to test HTTP requests in expect.
  await state.httpClient.get('http://fake.api').then((_) {}, onError: (_) {});

  return new v0.CalculationResponse(new v0.CalculationResult(r.a + r.b));
}

/// Subtraction calculator
Future<v0.CalculationResponse> calcSubtractHandler(
    v0.CalculationRequest r, EmptyState state) async {
  return new v0.CalculationResponse(new v0.CalculationResult(r.a - r.b));
}

/// Multiplication calculator
Future<v0.CalculationResponse> calcMultiplyHandler(
    v0.CalculationRequest r, EmptyState state) async {
  return new v0.CalculationResponse(new v0.CalculationResult(r.a * r.b));
}

/// Division calculator
Future<v0.CalculationResponse> calcDivideHandler(
    v0.CalculationRequest r, EmptyState state) async {
  // Delay by 1 second to run through future callbacks in Responses.forEach.
  return new Future<OmnibusResponse>.delayed(new Duration(seconds: 1),
      () => new v0.CalculationResponse(new v0.CalculationResult(r.a / r.b)));
}
